/**
 * \file Tree.cxx
 *
 * \date 30 mars 2011
 * \author Bruno Belarte
 */

#ifndef TREE_cxx_
#define TREE_cxx_

#include "Tree.h"

template <class T>
Tree<T>::
Tree()
{
	this->setRoot(T());
}

template <class T>
Tree<T>::
Tree(const T & t)
{
	this->setRoot(t);
}

template<class T>
typename Tree<T>::IteratorBase
Tree<T>::
add (const T & t)
{
	NodePointer node = NodePointer(new TreeNode<T>(t, m_Root));
	m_Root->children.push_back(node);
	return IteratorBase(node);
}

template<class T>
template<class iter>
iter
Tree<T>::
add (iter position, const T & t)
{
	NodePointer node = NodePointer(new TreeNode<T>(t, position.m_Node));
	position.m_Node->children.push_back(node);
	return iter(node);
}

template<class T>
typename Tree<T>::IteratorBase
Tree<T>::
append (Tree<T> & other)
{
	m_Root->children.push_back(other.m_Root);
	other.m_Root->parent = m_Root;
	return IteratorBase(other.m_Root);
}

template<class T>
template<class iter>
iter
Tree<T>::
append (iter position, Tree<T> & other)
{
	position.m_Node->children.push_back(other.m_Root);
	other.m_Root->parent = position.m_Node;
	return IteratorBase(other.m_Root);
}

template<class T>
template<class iter>
iter
Tree<T>::
erase_children (iter position)
{
	position.m_Node->children.clear();
	return position;
}

template<class T>
template<class iter>
iter
Tree<T>::
erase (iter position)
{
	iter result;

	if (position.m_Node->parent.expired())
	{
		m_Root->children.clear();
		m_Root->data = T();
		result = iter();
	} else {
		result = this->parent(position);
		std::find(result.m_Node->children.begin(), result.m_Node->children.end(), position.m_Node);
		result.m_Node->children.erase(std::find(result.m_Node->children.begin(), result.m_Node->children.end(), position.m_Node));
	}

	return result;
}

template<class T>
void
Tree<T>::
clear ()
{
	this->erase(m_Root);
}

template<class T>
template<class iter>
iter
Tree<T>::
parent (iter position)
{
	if (position.m_Node->parent.expired())
		return iter();

	return iter(NodePointer(position.m_Node->parent));
}

template<class T>
template<class iter>
iter
Tree<T>::
find (iter first, iter last, const T & t)
{
	while (first != last)
	{
		if (*first == t) break;
		++first;
	}
	return first;
}

template<class T>
TreeNode<T> *
Tree<T>::
getRoot()
{
	return m_Root.get();
}

template<class T>
Tree<T>
Tree<T>::
setRoot (const T & t)
{
	m_Root = NodePointer(new TreeNode<T>(t));
	return *this;
}

/*template<class T>
Tree<T>
Tree<T>::
setRoot (TreeNode<T> * node)
{
	m_Root = NodePoiter(node);
	return *this;
}*/

template<class T>
typename Tree<T>::PreOrderIterator
Tree<T>::
beginPre ()
{
	return PreOrderIterator(m_Root);
}

template<class T>
typename Tree<T>::PreOrderIterator
Tree<T>::
endPre ()
{
	return PreOrderIterator();
}

template<class T>
typename Tree<T>::BreadthFirstIterator
Tree<T>::
beginBF ()
{
	return BreadthFirstIterator(m_Root);
}

template<class T>
typename Tree<T>::BreadthFirstIterator
Tree<T>::
endBF ()
{
	return BreadthFirstIterator();
}

#endif // TREE_CXX_
