/**
 * \file TreeBreadthFirstIterator.cxx
 *
 * \date 31 mars 2011
 * \author Bruno Belarte
 */

#ifndef TREE_ITERATOR_CXX
#define TREE_ITERATOR_CXX

#include "Tree.h"

// IteratorBase

template<class T>
Tree<T>::IteratorBase::
IteratorBase () :
	m_Node()
{
}

template<class T>
Tree<T>::IteratorBase::
IteratorBase (NodePointer n) :
	m_Node(n)
{
}

template<class T>
T &
Tree<T>::IteratorBase::
operator* () const
{
	return m_Node->data;
}

template<class T>
T *
Tree<T>::IteratorBase::
operator-> () const
{
	return &(m_Node->data);
}

// PreOrderIterator

template<class T>
Tree<T>::PreOrderIterator::
PreOrderIterator () :
	IteratorBase()
{
}

template<class T>
Tree<T>::PreOrderIterator::
PreOrderIterator (NodePointer n) :
	IteratorBase(n)
{
}

template<class T>
Tree<T>::PreOrderIterator::
PreOrderIterator (const IteratorBase & other) :
	IteratorBase(other.m_Node)
{
}

template<class T>
bool
Tree<T>::PreOrderIterator::
operator== (const PreOrderIterator & other) const
{
	return (this->m_Node == other.m_Node);
}

template<class T>
bool
Tree<T>::PreOrderIterator::
operator!= (const PreOrderIterator & other) const
{
	return (this->m_Node != other.m_Node);
}

template<class T>
typename Tree<T>::PreOrderIterator &
Tree<T>::PreOrderIterator::
operator++ ()
{
	assert (this->m_Node != NULL);

	typename std::vector<NodePointer>::reverse_iterator begin	= this->m_Node->children.rbegin();
	typename std::vector<NodePointer>::reverse_iterator end		= this->m_Node->children.rend();

	while (begin != end)
	{
		m_Stack.push(*begin);
		++begin;
	}

	if (m_Stack.empty())
	{
		this->m_Node = NodePointer();
	} else {
		this->m_Node = m_Stack.top();
		m_Stack.pop();
	}

	return (*this);
}

template<class T>
typename Tree<T>::PreOrderIterator
Tree<T>::PreOrderIterator::
operator++ (int)
{
	PreOrderIterator copy = *this;
	++(*this);
	return copy;
}

template<class T>
typename Tree<T>::PreOrderIterator &
Tree<T>::PreOrderIterator::
operator+= (unsigned int count)
{
	while ((count > 0) && (this->m_Node != NULL))
	{
		++(*this);
		--count;
	}
	return (*this);
}

// BreadthFirstIterator

template<class T>
Tree<T>::BreadthFirstIterator::
BreadthFirstIterator () :
	IteratorBase()
{
}

template<class T>
Tree<T>::BreadthFirstIterator::
BreadthFirstIterator (NodePointer n) :
	IteratorBase(n)
{
	m_Queue.push(this->m_Node);
}

template<class T>
Tree<T>::BreadthFirstIterator::
BreadthFirstIterator (const IteratorBase & other) :
	IteratorBase(other.m_Node)
{
	m_Queue.push(this->m_Node);
}

template<class T>
bool
Tree<T>::BreadthFirstIterator::
operator== (const BreadthFirstIterator & other) const
{
	return (this->m_Node == other.m_Node);
}

template<class T>
bool
Tree<T>::BreadthFirstIterator::
operator!= (const BreadthFirstIterator & other) const
{
	return (this->m_Node != other.m_Node);
}

template<class T>
typename Tree<T>::BreadthFirstIterator &
Tree<T>::BreadthFirstIterator::
operator++ ()
{
	assert (this->m_Node != NULL);

	typename std::vector<NodePointer>::iterator begin	= this->m_Node->children.begin();
	typename std::vector<NodePointer>::iterator end		= this->m_Node->children.end();

	while (begin != end)
	{
		m_Queue.push(*begin);
		++begin;
	}

	m_Queue.pop();

	if (m_Queue.empty())
		this->m_Node = NodePointer();
	else
		this->m_Node = m_Queue.front();

	return (*this);
}

template<class T>
typename Tree<T>::BreadthFirstIterator
Tree<T>::BreadthFirstIterator::
operator++ (int)
{
	BreadthFirstIterator copy = *this;
	++(*this);
	return copy;
}

template<class T>
typename Tree<T>::BreadthFirstIterator &
Tree<T>::BreadthFirstIterator::
operator+= (unsigned int count)
{
	while ((count > 0) && (this->m_Node != NULL))
	{
		++(*this);
		--count;
	}
	return (*this);
}

#endif // TREE_ITERATOR_CXX
