#include <boost/test/unit_test.hpp>
#include "DataStructure/Graph/AdjacencyList.h"

struct AdjacencyListOutEdgeIteratorTest
{
	AdjacencyList<std::string, int> graph;

	std::string n1,n2,n3,n4,n5;
	int e1,e2,e3,e4,e5;

	AdjacencyListOutEdgeIteratorTest()
	{
		n1 = "n1";
		n2 = "n2";
		n3 = "n2";
		n4 = "n4";
		n5 = "n3";
		e1 = 1;
		e2 = 2;
		e3 = 2;
		e4 = 4;
		e5 = 3;
	}

	~AdjacencyListOutEdgeIteratorTest()
	{
	}

	void init_vertices()
	{
		BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
		BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n5),true);
	}

	void init_edges()
	{
		BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n5),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n4),true);
	}
};

BOOST_FIXTURE_TEST_SUITE(AdjacencyListOutEdgeIteratorTestSuite, AdjacencyListOutEdgeIteratorTest)

BOOST_AUTO_TEST_CASE(constructor)
{
	init_vertices();

	BOOST_CHECK_EQUAL(graph.contains(n1), true);

	AdjacencyList<std::string, int>::OutEdgeIterator begin = graph.begin(n1);
	AdjacencyList<std::string, int>::OutEdgeIterator end = graph.end(n1);

	BOOST_CHECK(begin == end);

	init_edges();

	begin = graph.begin(n1);
	end = graph.end(n1);
	AdjacencyList<std::string, int>::OutEdgeIterator iter(begin);
	AdjacencyList<std::string, int>::OutEdgeIterator iter2(&graph, n1);

	BOOST_CHECK(begin != end);
	BOOST_CHECK(begin == iter);
	BOOST_CHECK(iter != end);
	BOOST_CHECK(begin != end);
	BOOST_CHECK(iter2 != end);
	BOOST_CHECK(begin == iter2);

	iter2 = end;

	BOOST_CHECK(begin != end);
	BOOST_CHECK(begin == iter);
	BOOST_CHECK(iter != end);
	BOOST_CHECK(begin != end);
	BOOST_CHECK(iter2 == end);
	BOOST_CHECK(begin != iter2);
}

BOOST_AUTO_TEST_CASE(operator_plusplus)
{
	init_vertices();
	init_edges();

	AdjacencyList<std::string, int>::VertexIterator vbegin	= graph.begin();
	AdjacencyList<std::string, int>::VertexIterator vend	= graph.end();

	while (vbegin != vend)
	{
		BOOST_CHECK_EQUAL(graph.contains(*vbegin), true);

		AdjacencyList<std::string, int>::OutEdgeIterator begin = graph.begin(*vbegin);
		AdjacencyList<std::string, int>::OutEdgeIterator end = graph.end(*vbegin);

		while (begin != end)
		{
			BOOST_CHECK(begin != end);
			++begin;
		}

		BOOST_CHECK(begin == end);

		++vbegin;
	}
}

BOOST_AUTO_TEST_CASE(operator_star)
{
	std::string n9("n9");

	init_vertices();
	init_edges();

	AdjacencyList<std::string, int>::VertexIterator vbegin	= graph.begin();
	AdjacencyList<std::string, int>::VertexIterator vend	= graph.end();
	AdjacencyList<std::string, int>::OutEdgeIterator begin;
	AdjacencyList<std::string, int>::OutEdgeIterator end;

	BOOST_CHECK_EQUAL(graph.add_vertex(n9), true);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	begin	= graph.begin(n1);
	end		= graph.end(n1);
	BOOST_CHECK(begin != end);
	BOOST_CHECK_EQUAL((*begin).first, n2);
	BOOST_CHECK_EQUAL((*begin).second, e1);
	++begin;
	BOOST_CHECK_EQUAL((*begin).first, n4);
	BOOST_CHECK_EQUAL((*begin).second, e2);
	++begin;
	BOOST_CHECK(begin == end);

	BOOST_CHECK_EQUAL(graph.contains(n2), true);
	begin	= graph.begin(n2);
	end		= graph.end(n2);
	BOOST_CHECK(begin != end);
	BOOST_CHECK_EQUAL((*begin).first, n1);
	BOOST_CHECK_EQUAL((*begin).second, e1);
	++begin;
	BOOST_CHECK(begin == end);

	BOOST_CHECK_EQUAL(n2, n3);

	BOOST_CHECK_EQUAL(graph.contains(n4), true);
	begin	= graph.begin(n4);
	end		= graph.end(n4);
	BOOST_CHECK(begin != end);
	BOOST_CHECK_EQUAL((*begin).first, n1);
	BOOST_CHECK_EQUAL((*begin).second, e2);
	++begin;
	BOOST_CHECK_EQUAL((*begin).first, n4);
	BOOST_CHECK_EQUAL((*begin).second, e3);
	++begin;
	BOOST_CHECK_EQUAL((*begin).first, n5);
	BOOST_CHECK_EQUAL((*begin).second, e3);
	++begin;
	BOOST_CHECK(begin == end);

	BOOST_CHECK_EQUAL(graph.contains(n5), true);
	begin	= graph.begin(n5);
	end		= graph.end(n5);
	BOOST_CHECK(begin != end);
	BOOST_CHECK_EQUAL((*begin).first, n4);
	BOOST_CHECK_EQUAL((*begin).second, e3);
	++begin;
	BOOST_CHECK(begin == end);

	BOOST_CHECK_EQUAL(graph.contains(n9), true);
	begin	= graph.begin(n9);
	end		= graph.end(n9);
	BOOST_CHECK(begin == end);
}

BOOST_AUTO_TEST_SUITE_END()
