#include <boost/test/unit_test.hpp>
#include "DataStructure/Graph/AdjacencyList.h"

struct AdjacencyListTest
{
	AdjacencyList<std::string, int> graph;

	std::string n1,n2,n3,n4,n5;
	int e1,e2,e3,e4,e5;

	AdjacencyListTest()
	{
		n1 = "n1";
		n2 = "n2";
		n3 = "n2";
		n4 = "n4";
		n5 = "n3";
		e1 = 1;
		e2 = 2;
		e3 = 2;
		e4 = 4;
		e5 = 3;
	}

	~AdjacencyListTest()
	{
	}

	void init_vertices()
	{
		BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
		BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n5),true);
	}

	void init_edges()
	{
		BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n5),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n4),true);
	}
};

BOOST_FIXTURE_TEST_SUITE(AdjacencyListTestSuite, AdjacencyListTest)

BOOST_AUTO_TEST_CASE(add_vertex)
{
	BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n5),true);

	BOOST_CHECK_EQUAL(graph.add_vertex(n1),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n4),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n5),false);
}

BOOST_AUTO_TEST_CASE(remove_vertex)
{
	BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
	BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);

	BOOST_CHECK_EQUAL(graph.remove_vertex(n1),true);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n1),false);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n2),true);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n3),false);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n4),true);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n5),false);

	BOOST_CHECK_EQUAL(graph.add_vertex(n1), true);
}

BOOST_AUTO_TEST_CASE(replace_vertex)
{
	BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
	BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);

	BOOST_CHECK_EQUAL(graph.replace_vertex(n1,n2),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n1,n3),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n2,n3),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n2,n5),true);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n2,n5),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n1,n5),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n5,n1),false);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n4,n2),true);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n5,n2),false);
	
	BOOST_CHECK_EQUAL(graph.remove_vertex(n5), true);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n1, n5), true);
}

BOOST_AUTO_TEST_CASE(vertices_count)
{
	BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 1);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 2);
	BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 2);
	BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 3);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n2, n5), true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 3);
	BOOST_CHECK_EQUAL(graph.add_vertex(n2), true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 4);
	BOOST_CHECK_EQUAL(graph.remove_vertex(n1), true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 3);
	BOOST_CHECK_EQUAL(graph.replace_vertex(n4, n1), true);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 3);
}

BOOST_AUTO_TEST_CASE(contains)
{
	BOOST_CHECK_EQUAL(graph.contains(n2), false);
	BOOST_CHECK_EQUAL(graph.contains(n3), false);
	BOOST_CHECK_EQUAL(graph.contains(n4), false);

	BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), false);
	BOOST_CHECK_EQUAL(graph.contains(n3), false);
	BOOST_CHECK_EQUAL(graph.contains(n4), false);

	BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), true);
	BOOST_CHECK_EQUAL(graph.contains(n3), true);
	BOOST_CHECK_EQUAL(graph.contains(n4), false);

	BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), true);
	BOOST_CHECK_EQUAL(graph.contains(n3), true);
	BOOST_CHECK_EQUAL(graph.contains(n4), false);

	BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), true);
	BOOST_CHECK_EQUAL(graph.contains(n3), true);
	BOOST_CHECK_EQUAL(graph.contains(n4), true);
}

BOOST_AUTO_TEST_CASE(add_edge)
{
	std::string n6;
	n6 = "n6";

	init_vertices();

	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n2),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n2,n1),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n3),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e3,n1,n3),false);

	BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),true);
	BOOST_CHECK_EQUAL(graph.add_edge(e3,n1,n4),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e5,n4,n1),false);

	BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n5),true);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n4,n5),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e3,n5,n4),false);

	BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n4),true);
	BOOST_CHECK_EQUAL(graph.add_edge(e5,n4,n4),false);

	BOOST_CHECK_EQUAL(graph.add_edge(e5,n5,n6),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e4,n6,n5),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e5,n2,n6),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n6,n2),false);
}

BOOST_AUTO_TEST_CASE(remove_edge)
{
	std::string n6 = "n6";
	init_vertices();

	BOOST_CHECK_EQUAL(graph.remove_edge(n1,n2),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
	BOOST_CHECK_EQUAL(graph.remove_edge(n1,n2),true);
	BOOST_CHECK_EQUAL(graph.remove_edge(n1,n2),false);

	BOOST_CHECK_EQUAL(graph.remove_edge(n2,n4),false);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n2,n4),true);
	BOOST_CHECK_EQUAL(graph.remove_edge(n4,n2),true);
	BOOST_CHECK_EQUAL(graph.remove_edge(n4,n2),false);

	BOOST_CHECK_EQUAL(graph.remove_edge(n3,n6),false);
}

BOOST_AUTO_TEST_CASE(edge_count)
{
	std::string n6 = "n6";
	init_vertices();

	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),1);
	BOOST_CHECK_EQUAL(graph.remove_edge(n1,n2),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),1);
	BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),2);
	BOOST_CHECK_EQUAL(graph.add_edge(e3,n1,n5),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),3);
	BOOST_CHECK_EQUAL(graph.add_edge(e4,n3,n5),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),4);
	BOOST_CHECK_EQUAL(graph.remove_edge(n1,n5),true);
	BOOST_CHECK_EQUAL(graph.edges_count(),3);
	BOOST_CHECK_EQUAL(graph.add_edge(e5,n1,n6),false);
	BOOST_CHECK_EQUAL(graph.edges_count(),3);
}

BOOST_AUTO_TEST_CASE(clear)
{
	BOOST_CHECK_EQUAL(graph.vertices_count(),0);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	init_vertices();
	BOOST_CHECK_EQUAL(graph.vertices_count(),4);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	init_edges();
	BOOST_CHECK_EQUAL(graph.vertices_count(),4);
	BOOST_CHECK_EQUAL(graph.edges_count(),4);
	graph.clear();
	BOOST_CHECK_EQUAL(graph.vertices_count(),0);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
}

BOOST_AUTO_TEST_CASE(copy_constructor)
{
	BOOST_CHECK_EQUAL(graph.vertices_count(),0);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	init_vertices();
	init_edges();
	BOOST_CHECK_EQUAL(graph.vertices_count(),4);
	BOOST_CHECK_EQUAL(graph.edges_count(),4);

	AdjacencyList<std::string, int> copy(graph);	
	BOOST_CHECK_EQUAL(copy.vertices_count(),4);
	BOOST_CHECK_EQUAL(copy.edges_count(),4);
	BOOST_CHECK_EQUAL(copy.contains(n1),true);
	BOOST_CHECK_EQUAL(copy.contains(n2),true);
	BOOST_CHECK_EQUAL(copy.contains(n3),true);
	BOOST_CHECK_EQUAL(copy.contains(n4),true);
	BOOST_CHECK_EQUAL(copy.contains(n5),true);
	graph.clear();
	BOOST_CHECK_EQUAL(copy.vertices_count(),4);
	BOOST_CHECK_EQUAL(copy.edges_count(),4);
	BOOST_CHECK_EQUAL(copy.contains(n1),true);
	BOOST_CHECK_EQUAL(copy.contains(n2),true);
	BOOST_CHECK_EQUAL(copy.contains(n3),true);
	BOOST_CHECK_EQUAL(copy.contains(n4),true);
	BOOST_CHECK_EQUAL(copy.contains(n5),true);
}

BOOST_AUTO_TEST_CASE(operator_equal)
{
	BOOST_CHECK_EQUAL(graph.vertices_count(),0);
	BOOST_CHECK_EQUAL(graph.edges_count(),0);
	init_vertices();
	init_edges();
	BOOST_CHECK_EQUAL(graph.vertices_count(),4);
	BOOST_CHECK_EQUAL(graph.edges_count(),4);

	AdjacencyList<std::string, int> copy;	
	BOOST_CHECK_EQUAL(copy.vertices_count(),0);
	BOOST_CHECK_EQUAL(copy.edges_count(),0);
	copy = graph;
	BOOST_CHECK_EQUAL(copy.vertices_count(),4);
	BOOST_CHECK_EQUAL(copy.edges_count(),4);
	BOOST_CHECK_EQUAL(copy.contains(n1),true);
	BOOST_CHECK_EQUAL(copy.contains(n2),true);
	BOOST_CHECK_EQUAL(copy.contains(n3),true);
	BOOST_CHECK_EQUAL(copy.contains(n4),true);
	BOOST_CHECK_EQUAL(copy.contains(n5),true);
	graph.clear();
	BOOST_CHECK_EQUAL(copy.vertices_count(),4);
	BOOST_CHECK_EQUAL(copy.edges_count(),4);
	BOOST_CHECK_EQUAL(copy.contains(n1),true);
	BOOST_CHECK_EQUAL(copy.contains(n2),true);
	BOOST_CHECK_EQUAL(copy.contains(n3),true);
	BOOST_CHECK_EQUAL(copy.contains(n4),true);
	BOOST_CHECK_EQUAL(copy.contains(n5),true);
}

BOOST_AUTO_TEST_SUITE_END()
