#include <boost/test/unit_test.hpp>
#include "DataStructure/Graph/AdjacencyList.h"

struct AdjacencyListVertexIteratorTest
{
	AdjacencyList<std::string, int> graph;

	std::string n1,n2,n3,n4,n5;
	int e1,e2,e3,e4,e5;

	AdjacencyListVertexIteratorTest()
	{
		n1 = "n1";
		n2 = "n2";
		n3 = "n2";
		n4 = "n4";
		n5 = "n3";
		e1 = 1;
		e2 = 2;
		e3 = 2;
		e4 = 4;
		e5 = 3;
	}

	~AdjacencyListVertexIteratorTest()
	{
	}

	void init_vertices()
	{
		BOOST_CHECK_EQUAL(graph.add_vertex(n1),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n2),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n3),false);
		BOOST_CHECK_EQUAL(graph.add_vertex(n4),true);
		BOOST_CHECK_EQUAL(graph.add_vertex(n5),true);
	}

	void init_edges()
	{
		BOOST_CHECK_EQUAL(graph.add_edge(e1,n1,n2),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e2,n1,n4),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n5),true);
		BOOST_CHECK_EQUAL(graph.add_edge(e3,n4,n4),true);
	}
};

BOOST_FIXTURE_TEST_SUITE(AdjacencyListVertexIteratorTestSuite, AdjacencyListVertexIteratorTest)

BOOST_AUTO_TEST_CASE(constructors)
{
	init_vertices();

	AdjacencyList<std::string, int>::VertexIterator iter;
	AdjacencyList<std::string, int>::VertexIterator iter2 = graph.begin();
	AdjacencyList<std::string, int>::VertexIterator iter3 = graph.end();

	BOOST_CHECK(iter != iter2);
	BOOST_CHECK(iter != iter3);
	BOOST_CHECK(iter2 != iter3);

	AdjacencyList<std::string, int>::VertexIterator iter4(iter3);

	BOOST_CHECK(iter4 != iter);
	BOOST_CHECK(iter4 != iter2);
	BOOST_CHECK(iter4 == iter3);

	iter = iter2;

	BOOST_CHECK(iter == iter2);
	BOOST_CHECK(iter != iter3);
	BOOST_CHECK(iter != iter4);
	BOOST_CHECK(iter2 != iter3);
	BOOST_CHECK(iter2 != iter4);
	BOOST_CHECK(iter3 == iter4);
}

BOOST_AUTO_TEST_CASE(operator_plusplus)
{
	init_vertices();
	int i = 0;

	AdjacencyList<std::string, int>::VertexIterator begin	= graph.begin();
	AdjacencyList<std::string, int>::VertexIterator end	= graph.end();
	AdjacencyList<std::string, int>::VertexIterator iter(begin); 

	BOOST_CHECK(begin == iter);
	BOOST_CHECK(begin != end);
	BOOST_CHECK(iter != end);

	while (iter != end)
	{
		BOOST_CHECK(iter != end);
		++i;
		++iter;
	}

	BOOST_CHECK(begin != end);
	BOOST_CHECK(iter == end);
	BOOST_CHECK(begin != iter);

	BOOST_CHECK_EQUAL(i, graph.vertices_count());
}

BOOST_AUTO_TEST_CASE(operator_star)
{
	std::string n6("n6");
	std::string n7("n7");

	init_vertices();

	AdjacencyList<std::string, int>::VertexIterator begin = graph.begin();
	AdjacencyList<std::string, int>::VertexIterator end	= graph.end();
	AdjacencyList<std::string, int>::VertexIterator iter(begin); 

	BOOST_CHECK(iter == begin);
	BOOST_CHECK_EQUAL(*iter,n1);
	BOOST_CHECK_EQUAL(*iter,"n1");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n2);
	BOOST_CHECK_EQUAL(*iter,"n2");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n4);
	BOOST_CHECK_EQUAL(*iter,"n4");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n5);
	BOOST_CHECK_EQUAL(*iter,"n3");
	++iter;
	BOOST_CHECK(iter == end);

	iter.begin();

	BOOST_CHECK(iter == begin);
	BOOST_CHECK_EQUAL(*iter,n1);
	BOOST_CHECK_EQUAL(*iter,"n1");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n2);
	BOOST_CHECK_EQUAL(*iter,"n2");
	BOOST_CHECK_EQUAL(graph.contains(*iter), true);
	BOOST_CHECK_EQUAL(graph.remove_vertex(*iter), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), false);

	iter.begin();
	BOOST_CHECK_EQUAL(*iter, n1);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n4);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n5);
	++iter;
	BOOST_CHECK(iter == end);

	BOOST_CHECK_EQUAL(graph.add_vertex(n6), true);
	
	iter.begin();
	BOOST_CHECK_EQUAL(*iter, n1);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n4);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n5);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n6);
	++iter;
	BOOST_CHECK(iter == end);

	BOOST_CHECK_EQUAL(graph.contains(n1), true);
	BOOST_CHECK_EQUAL(graph.contains(n2), false);
	BOOST_CHECK_EQUAL(graph.contains(n3), false);
	BOOST_CHECK_EQUAL(graph.contains(n4), true);
	BOOST_CHECK_EQUAL(graph.contains(n5), true);
	BOOST_CHECK_EQUAL(graph.contains(n6), true);
	BOOST_CHECK_EQUAL(graph.contains(n7), false);

	iter.begin();
	++iter;
	++iter;
	BOOST_CHECK_EQUAL(*iter, n5);
	BOOST_CHECK_EQUAL(graph.replace_vertex(*iter, n7), true);
	BOOST_CHECK_EQUAL(graph.contains(n7), true);
	BOOST_CHECK_EQUAL(graph.contains(n5), false);

	iter.begin();
	BOOST_CHECK_EQUAL(*iter,n1);
	++iter;
	BOOST_CHECK_EQUAL(*iter,n4);
	++iter;
	BOOST_CHECK_EQUAL(*iter,n7);
	++iter;
	BOOST_CHECK_EQUAL(*iter,n6);
	++iter;
	BOOST_CHECK(iter == end);
}

BOOST_AUTO_TEST_CASE(set)
{
	std::string n6("n6");
	std::string n7("n7");

	init_vertices();
	init_edges();

	AdjacencyList<std::string, int>::VertexIterator begin = graph.begin();
	AdjacencyList<std::string, int>::VertexIterator end	= graph.end();
	AdjacencyList<std::string, int>::VertexIterator iter(begin); 

	BOOST_CHECK_EQUAL(*iter, n1);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n2);
	iter.set(n2);
	BOOST_CHECK_EQUAL(*iter, n2);
	iter.set(n1);
	BOOST_CHECK_EQUAL(*iter, n2);
	iter.set(n6);
	BOOST_CHECK_EQUAL(*iter, n6);
	
	iter.begin();

	BOOST_CHECK(iter == begin);
	BOOST_CHECK_EQUAL(*iter,n1);
	BOOST_CHECK_EQUAL(*iter,"n1");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n6);
	BOOST_CHECK_EQUAL(*iter,"n6");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n4);
	BOOST_CHECK_EQUAL(*iter,"n4");
	++iter;
	BOOST_CHECK_EQUAL(*iter,n5);
	BOOST_CHECK_EQUAL(*iter,"n3");
	++iter;
	BOOST_CHECK(iter == end);
	
	graph.add_vertex(n7);

	iter = graph.begin();
	end = graph.end();
	BOOST_CHECK_EQUAL(*iter, n1);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n6);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n4);
	BOOST_CHECK_EQUAL(graph.replace_vertex(*iter, n2), true);
	BOOST_CHECK_EQUAL(*iter, n2);
	BOOST_CHECK_EQUAL(graph.vertices_count(), 5);
	++iter;
	BOOST_CHECK_EQUAL(*iter, n5);
	++iter;
	BOOST_CHECK_EQUAL(*iter, "n7");
	++iter;
	BOOST_CHECK(iter == end);
}

BOOST_AUTO_TEST_SUITE_END()
